@@ -530,151 +530,6 @@ TEST_CASE_METHOD(
   CHECK(error_str.find("Cannot load a single tile") != std::string::npos);
 }
 
-TEST_CASE_METHOD(
-    CSparseGlobalOrderFx,
-    "Sparse global order reader: qc budget too small",
-    "[sparse-global-order][qc-budget][too-small]") {
-  // Create default array.
-  reset_config();
-  create_default_array_1d();
-
-  bool use_subarray = false;
-  SECTION("- No subarray") {
-    use_subarray = false;
-  }
-  SECTION("- Subarray") {
-    use_subarray = true;
-  }
-
-  // Write a fragment.
-  int coords[] = {1, 2, 3, 4, 5};
-  uint64_t coords_size = sizeof(coords);
-  int data[] = {1, 2, 3, 4, 5};
-  uint64_t data_size = sizeof(data);
-  write_1d_fragment(coords, &coords_size, data, &data_size);
-
-  // One qc tile (8) will be bigger than the budget (5).
-  total_budget_ = "10000";
-  ratio_query_condition_ = "0.0005";
-  update_config();
-
-  // Try to read.
-  int coords_r[5];
-  int data_r[5];
-  uint64_t coords_r_size = sizeof(coords_r);
-  uint64_t data_r_size = sizeof(data_r);
-  auto rc =
-      read(use_subarray, true, coords_r, &coords_r_size, data_r, &data_r_size);
-  CHECK(rc == TILEDB_ERR);
-
-  // Check we hit the correct error.
-  tiledb_error_t* error = NULL;
-  rc = tiledb_ctx_get_last_error(ctx_, &error);
-  CHECK(rc == TILEDB_OK);
-
-  const char* msg;
-  rc = tiledb_error_message(error, &msg);
-  CHECK(rc == TILEDB_OK);
-
-  std::string error_str(msg);
-  CHECK(error_str.find("Cannot load a single tile") != std::string::npos);
-}
-
-TEST_CASE_METHOD(
-    CSparseGlobalOrderFx,
-    "Sparse global order reader: qc budget forcing one tile at a time",
-    "[sparse-global-order][small-qc-budget]") {
-  // Create default array.
-  reset_config();
-  create_default_array_1d();
-
-  bool use_subarray = false;
-  int num_frags = 0;
-  SECTION("- No subarray") {
-    use_subarray = false;
-    SECTION("- Two fragments") {
-      num_frags = 2;
-    }
-  }
-  SECTION("- Subarray") {
-    use_subarray = true;
-    SECTION("- One fragment") {
-      num_frags = 1;
-    }
-    SECTION("- Two fragments") {
-      num_frags = 2;
-    }
-  }
-
-  for (int i = 0; i < num_frags; i++) {
-    // Write a fragment.
-    int coords[] = {1 + i * 5, 2 + i * 5, 3 + i * 5, 4 + i * 5, 5 + i * 5};
-    uint64_t coords_size = sizeof(coords);
-    int data[] = {1 + i * 5, 2 + i * 5, 3 + i * 5, 4 + i * 5, 5 + i * 5};
-    uint64_t data_size = sizeof(data);
-    write_1d_fragment(coords, &coords_size, data, &data_size);
-  }
-
-  // Two qc tile (16) will be bigger than the budget (10).
-  total_budget_ = "10000";
-  ratio_query_condition_ = num_frags == 1 ? "0.001" : "0.002";
-  update_config();
-
-  tiledb_array_t* array = nullptr;
-  tiledb_query_t* query = nullptr;
-
-  uint32_t rc;
-  uint64_t coords_r_size;
-  uint64_t data_r_size;
-
-  // Try to read.
-  int coords_r[10];
-  int data_r[10];
-  coords_r_size = sizeof(coords_r);
-  data_r_size = sizeof(data_r);
-
-  rc = read(
-      use_subarray,
-      true,
-      coords_r,
-      &coords_r_size,
-      data_r,
-      &data_r_size,
-      &query,
-      &array);
-  CHECK(rc == TILEDB_OK);
-
-  // Check the internal loop count against expected value.
-  auto stats =
-      ((sm::SparseGlobalOrderReader*)query->query_->strategy())->stats();
-  REQUIRE(stats != nullptr);
-  auto counters = stats->counters();
-  REQUIRE(counters != nullptr);
-  auto loop_num =
-      counters->find("Context.StorageManager.Query.Reader.loop_num");
-  CHECK(2 + (uint64_t)num_frags == loop_num->second);
-
-  // Check incomplete query status.
-  tiledb_query_status_t status;
-  tiledb_query_get_status(ctx_, query, &status);
-  CHECK(status == TILEDB_COMPLETED);
-
-  // Should only read one tile (2 values).
-  CHECK(uint64_t(num_frags * 20) == data_r_size);
-  CHECK(uint64_t(num_frags * 20) == coords_r_size);
-
-  int coords_c[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
-  int data_c[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
-  CHECK(!std::memcmp(coords_c, coords_r, coords_r_size));
-  CHECK(!std::memcmp(data_c, data_r, data_r_size));
-
-  // Clean up.
-  rc = tiledb_array_close(ctx_, array);
-  CHECK(rc == TILEDB_OK);
-  tiledb_array_free(&array);
-  tiledb_query_free(&query);
-}
-
 TEST_CASE_METHOD(
     CSparseGlobalOrderFx,
     "Sparse global order reader: qc removes full tile",
@@ -793,7 +648,7 @@ TEST_CASE_METHOD(
   uint64_t data_r_size = sizeof(data_r);
 
   auto rc =
-      read(use_subarray, true, coords_r, &coords_r_size, data_r, &data_r_size);
+      read(use_subarray, false, coords_r, &coords_r_size, data_r, &data_r_size);
   CHECK(rc == TILEDB_OK);
 
   // Should read (6 values).
