@@ -608,40 +608,43 @@ std::pair<Status, optional<std::string>> libmagic_get_mime(
     void* data, uint64_t size) {
   magic_t magic = magic_open(MAGIC_MIME_TYPE);
   if (tiledb::sm::magic_dict::magic_mgc_embedded_load(magic)) {
+    std::string errmsg(magic_error(magic));
     magic_close(magic);
-    return {
-        Status_Error(
-            std::string("Cannot load magic database - ") + magic_error(magic)),
-        nullopt};
+    return {Status_Error(std::string("Cannot load magic database - ") + errmsg),
+            nullopt};
   }
   auto rv = magic_buffer(magic, data, size);
   if (!rv) {
-    return {
-        Status_Error(
-            std::string("Cannot get the mime type - ") + magic_error(magic)),
-        nullopt};
+    std::string errmsg(magic_error(magic));
+    magic_close(magic);
+    return {Status_Error(std::string("Cannot get the mime type - ") + errmsg),
+            nullopt};
   }
-  return {Status::Ok(), rv};
+  std::string mime(rv);
+  magic_close(magic);
+  return {Status::Ok(), mime};
 }
 
 std::pair<Status, optional<std::string>> libmagic_get_mime_encoding(
     void* data, uint64_t size) {
   magic_t magic = magic_open(MAGIC_MIME_ENCODING);
   if (tiledb::sm::magic_dict::magic_mgc_embedded_load(magic)) {
+    std::string errmsg(magic_error(magic));
     magic_close(magic);
-    return {
-        Status_Error(
-            std::string("Cannot load magic database - ") + magic_error(magic)),
-        nullopt};
+    return {Status_Error(std::string("Cannot load magic database - ") + errmsg),
+            nullopt};
   }
   auto rv = magic_buffer(magic, data, size);
   if (!rv) {
-    return {Status_Error(
-                std::string("Cannot get the mime encoding - ") +
-                magic_error(magic)),
-            nullopt};
+    std::string errmsg(magic_error(magic));
+    magic_close(magic);
+    return {
+        Status_Error(std::string("Cannot get the mime encoding - ") + errmsg),
+        nullopt};
   }
-  return {Status::Ok(), rv};
+  std::string mime(rv);
+  magic_close(magic);
+  return {Status::Ok(), mime};
 }
 
 bool libmagic_file_is_compressed(void* data, uint64_t size) {
@@ -661,10 +664,11 @@ bool libmagic_file_is_compressed(void* data, uint64_t size) {
   }
   auto rv = magic_buffer(magic, data, size);
   if (!rv) {
+    magic_close(magic);
     return true;
   }
-
-  std::string mime(magic_buffer(magic, data, size));
+  std::string mime(rv);
+  magic_close(magic);
 
   return compressed_mime_types.find(mime) != compressed_mime_types.end();
 }
