@@ -31,11 +31,16 @@
  */
 #ifdef _WIN32
 
-#include "tiledb/sm/filesystem/win.h"
+#include "win.h"
+#include "path_win.h"
+#include "tiledb/common/filesystem/directory_entry.h"
 #include "tiledb/common/heap_memory.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/misc/constants.h"
+#include "tiledb/sm/misc/tdb_math.h"
 #include "tiledb/sm/misc/utils.h"
+#include "uri.h"
 
 #if !defined(NOMINMAX)
 #define NOMINMAX  // suppress definition of min/max macros in Windows headers
@@ -50,6 +55,7 @@
 #include <sstream>
 
 using namespace tiledb::common;
+using tiledb::common::filesystem::directory_entry;
 
 namespace tiledb {
 namespace sm {
@@ -81,16 +87,19 @@ std::string Win::abs_path(const std::string& path) {
   if (path.length() == 0) {
     return current_dir();
   }
-  std::string full_path;
-  if (PathIsRelative(path.c_str())) {
-    full_path = current_dir() + "\\" + path;
+  std::string full_path(path_win::slashes_to_backslashes(path));
+  // If some problem leads here, note the following
+  // PathIsRelative("/") unexpectedly returns true.
+  // PathIsRelative("c:somedir\somesubdir") unexpectedly returns false
+  if (PathIsRelative(full_path.c_str())) {
+    full_path = current_dir() + "\\" + full_path;
   } else {
     full_path = path;
   }
   char result[MAX_PATH];
   std::string str_result;
   if (PathCanonicalize(result, full_path.c_str()) == FALSE) {
-    LOG_STATUS(Status::IOError(std::string("Cannot canonicalize path.")));
+    LOG_STATUS(Status_IOError(std::string("Cannot canonicalize path.")));
   } else {
     str_result = result;
   }
@@ -99,12 +108,12 @@ std::string Win::abs_path(const std::string& path) {
 
 Status Win::create_dir(const std::string& path) const {
   if (is_dir(path)) {
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         std::string("Cannot create directory '") + path +
         "'; Directory already exists"));
   }
   if (CreateDirectory(path.c_str(), nullptr) == 0) {
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         std::string("Cannot create directory '") + path +
         "': " + get_last_error_msg()));
   }
@@ -125,7 +134,7 @@ Status Win::touch(const std::string& filename) const {
       FILE_ATTRIBUTE_NORMAL,
       nullptr);
   if (file_h == INVALID_HANDLE_VALUE || CloseHandle(file_h) == 0) {
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         std::string("Failed to create file '") + filename + "'"));
   }
   return Status::Ok();
@@ -136,8 +145,7 @@ std::string Win::current_dir() {
   unsigned long length = GetCurrentDirectory(0, nullptr);
   char* path = (char*)tdb_malloc(length * sizeof(char));
   if (path == nullptr || GetCurrentDirectory(length, path) == 0) {
-    LOG_STATUS(
-        Status::IOError(std::string("Failed to get current directory.")));
+    LOG_STATUS(Status_IOError(std::string("Failed to get current directory.")));
   }
   dir = path;
   tdb_free(path);
@@ -193,15 +201,15 @@ err:
   if (find_h != INVALID_HANDLE_VALUE) {
     FindClose(find_h);
   }
-  return LOG_STATUS(Status::IOError(
-      std::string("Failed to remove directory '" + path + "'")));
+  return LOG_STATUS(
+      Status_IOError(std::string("Failed to remove directory '" + path + "'")));
 }
 
 Status Win::remove_dir(const std::string& path) const {
   if (is_dir(path)) {
     return recursively_remove_directory(path);
   } else {
-    return LOG_STATUS(Status::IOError(std::string(
+    return LOG_STATUS(Status_IOError(std::string(
         "Failed to delete path '" + path + "'; not a valid path.")));
   }
 }
@@ -209,7 +217,7 @@ Status Win::remove_dir(const std::string& path) const {
 Status Win::remove_file(const std::string& path) const {
   if (!DeleteFile(path.c_str())) {
     return LOG_STATUS(
-        Status::IOError(std::string("Failed to delete file '" + path + "'")));
+        Status_IOError(std::string("Failed to delete file '" + path + "'")));
   }
   return Status::Ok();
 }
@@ -225,12 +233,12 @@ Status Win::file_size(const std::string& path, uint64_t* size) const {
       FILE_ATTRIBUTE_NORMAL,
       NULL);
   if (file_h == INVALID_HANDLE_VALUE) {
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         std::string("Failed to get file size for '" + path + "'")));
   }
   if (!GetFileSizeEx(file_h, &nbytes)) {
     CloseHandle(file_h);
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         std::string("Failed to get file size for '" + path + "'")));
   }
   *size = nbytes.QuadPart;
@@ -238,53 +246,10 @@ Status Win::file_size(const std::string& path, uint64_t* size) const {
   return Status::Ok();
 }
 
-Status Win::filelock_lock(
-    const std::string& filename, filelock_t* fd, bool shared) const {
-  HANDLE file_h = CreateFile(
-      filename.c_str(),
-      GENERIC_READ | GENERIC_WRITE,
-      FILE_SHARE_READ | FILE_SHARE_WRITE,
-      NULL,
-      OPEN_EXISTING,
-      FILE_ATTRIBUTE_NORMAL,
-      NULL);
-  if (file_h == INVALID_HANDLE_VALUE) {
-    return LOG_STATUS(Status::IOError(
-        std::string("Failed to lock '" + filename + "'; CreateFile error")));
-  }
-  OVERLAPPED overlapped = {0, 0, {{0, 0}}, 0};
-  if (LockFileEx(
-          file_h,
-          shared ? 0 : LOCKFILE_EXCLUSIVE_LOCK,
-          0,
-          MAXDWORD,
-          MAXDWORD,
-          &overlapped) == 0) {
-    CloseHandle(file_h);
-    *fd = INVALID_FILELOCK;
-    return LOG_STATUS(Status::IOError(
-        std::string("Failed to lock '" + filename + "'; LockFile error")));
-  }
-
-  *fd = file_h;
-  return Status::Ok();
-}
-
-Status Win::filelock_unlock(filelock_t fd) const {
-  OVERLAPPED overlapped = {0, 0, {{0, 0}}, 0};
-  if (UnlockFileEx(fd, 0, MAXDWORD, MAXDWORD, &overlapped) == 0) {
-    CloseHandle(fd);
-    return LOG_STATUS(
-        Status::IOError(std::string("Failed to unlock file lock")));
-  }
-  CloseHandle(fd);
-  return Status::Ok();
-}
-
 Status Win::init(const Config& config, ThreadPool* vfs_thread_pool) {
   if (vfs_thread_pool == nullptr) {
     return LOG_STATUS(
-        Status::VFSError("Cannot initialize with null thread pool"));
+        Status_VFSError("Cannot initialize with null thread pool"));
   }
 
   config_ = config;
@@ -302,9 +267,23 @@ bool Win::is_file(const std::string& path) const {
 }
 
 Status Win::ls(const std::string& path, std::vector<std::string>* paths) const {
+  auto&& [st, entries] = ls_with_sizes(URI(path));
+  RETURN_NOT_OK(st);
+
+  for (auto& fs : *entries) {
+    paths->emplace_back(fs.path().native());
+  }
+
+  return Status::Ok();
+}
+
+tuple<Status, optional<std::vector<directory_entry>>> Win::ls_with_sizes(
+    const URI& uri) const {
+  auto path = uri.to_path();
   bool ends_with_slash = path.length() > 0 && path[path.length() - 1] == '\\';
   const std::string glob = path + (ends_with_slash ? "*" : "\\*");
   WIN32_FIND_DATA find_data;
+  std::vector<directory_entry> entries;
 
   // Get first file in directory.
   HANDLE find_h = FindFirstFileEx(
@@ -324,7 +303,14 @@ Status Win::ls(const std::string& path, std::vector<std::string>* paths) const {
         strcmp(find_data.cFileName, "..") != 0) {
       std::string file_path =
           path + (ends_with_slash ? "" : "\\") + find_data.cFileName;
-      paths->push_back(file_path);
+      if (is_dir(file_path)) {
+        entries.emplace_back(file_path, 0);
+      } else {
+        ULARGE_INTEGER size;
+        size.LowPart = find_data.nFileSizeLow;
+        size.HighPart = find_data.nFileSizeHigh;
+        entries.emplace_back(file_path, size.QuadPart);
+      }
     }
 
     // Next find result.
@@ -334,20 +320,22 @@ Status Win::ls(const std::string& path, std::vector<std::string>* paths) const {
   }
 
   FindClose(find_h);
-  return Status::Ok();
+  return {Status::Ok(), entries};
 
 err:
   if (find_h != INVALID_HANDLE_VALUE) {
     FindClose(find_h);
   }
-  return LOG_STATUS(Status::IOError(std::string("Failed to list directory.")));
+  std::string errmsg("Failed to list directory \"" + path + "\"");
+  auto st = LOG_STATUS(Status_IOError(errmsg));
+  return {st, nullopt};
 }
 
 Status Win::move_path(
     const std::string& old_path, const std::string& new_path) const {
   if (MoveFileEx(
           old_path.c_str(), new_path.c_str(), MOVEFILE_REPLACE_EXISTING) == 0) {
-    return LOG_STATUS(Status::IOError(std::string(
+    return LOG_STATUS(Status_IOError(std::string(
         "Failed to rename '" + old_path + "' to '" + new_path + "'.")));
   }
   return Status::Ok();
@@ -369,7 +357,7 @@ Status Win::read(
       FILE_ATTRIBUTE_NORMAL,
       NULL);
   if (file_h == INVALID_HANDLE_VALUE) {
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         "Cannot read from file '" + path + "'; File opening error"));
   }
 
@@ -377,7 +365,7 @@ Status Win::read(
   offset_lg_int.QuadPart = offset;
   if (SetFilePointerEx(file_h, offset_lg_int, NULL, FILE_BEGIN) == 0) {
     CloseHandle(file_h);
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         "Cannot read from file '" + path + "'; File seek error"));
   }
 
@@ -385,12 +373,12 @@ Status Win::read(
   if (ReadFile(file_h, buffer, nbytes, &num_bytes_read, NULL) == 0 ||
       num_bytes_read != nbytes) {
     CloseHandle(file_h);
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         "Cannot read from file '" + path + "'; File read error"));
   }
 
   if (CloseHandle(file_h) == 0) {
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         "Cannot read from file '" + path + "'; File closing error"));
   }
 
@@ -414,17 +402,17 @@ Status Win::sync(const std::string& path) const {
       NULL);
   if (file_h == INVALID_HANDLE_VALUE) {
     return LOG_STATUS(
-        Status::IOError("Cannot sync file '" + path + "'; File opening error"));
+        Status_IOError("Cannot sync file '" + path + "'; File opening error"));
   }
 
   if (FlushFileBuffers(file_h) == 0) {
     CloseHandle(file_h);
     return LOG_STATUS(
-        Status::IOError("Cannot sync file '" + path + "'; Sync error"));
+        Status_IOError("Cannot sync file '" + path + "'; Sync error"));
   }
 
   if (CloseHandle(file_h) == 0) {
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         "Cannot read from file '" + path + "'; File closing error"));
   }
 
@@ -455,15 +443,15 @@ Status Win::write(
       FILE_ATTRIBUTE_NORMAL,
       NULL);
   if (file_h == INVALID_HANDLE_VALUE) {
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         "Cannot write to file '" + path + "'; File opening error"));
   }
   // Get the current file size.
   LARGE_INTEGER file_size_lg_int;
   if (!GetFileSizeEx(file_h, &file_size_lg_int)) {
     CloseHandle(file_h);
-    return LOG_STATUS(Status::IOError(
-        "Cannot write to file '" + path + "'; File size error"));
+    return LOG_STATUS(
+        Status_IOError("Cannot write to file '" + path + "'; File size error"));
   }
   uint64_t file_offset = file_size_lg_int.QuadPart;
   // Ensure that each thread is responsible for at least min_parallel_size
@@ -474,7 +462,7 @@ Status Win::write(
     if (!write_at(file_h, file_offset, buffer, buffer_size).ok()) {
       CloseHandle(file_h);
       return LOG_STATUS(
-          Status::IOError(std::string("Cannot write to file '") + path));
+          Status_IOError(std::string("Cannot write to file '") + path));
     }
   } else {
     std::vector<ThreadPool::Task> results;
@@ -497,12 +485,12 @@ Status Win::write(
       CloseHandle(file_h);
       std::stringstream errmsg;
       errmsg << "Cannot write to file '" << path << "'; " << st.message();
-      return LOG_STATUS(Status::IOError(errmsg.str()));
+      return LOG_STATUS(Status_IOError(errmsg.str()));
     }
   }
   // Always close the handle.
   if (CloseHandle(file_h) == 0) {
-    return LOG_STATUS(Status::IOError(
+    return LOG_STATUS(Status_IOError(
         "Cannot write to file '" + path + "'; File closing error"));
   }
   return st;
@@ -534,7 +522,7 @@ Status Win::write_at(
             &bytes_written,
             &ov) == 0 ||
         bytes_written != constants::max_write_bytes) {
-      return LOG_STATUS(Status::IOError(std::string(
+      return LOG_STATUS(Status_IOError(std::string(
           "Cannot write to file; File writing error: " +
           get_last_error_msg())));
     }
@@ -551,73 +539,12 @@ Status Win::write_at(
           file_h, byte_buffer + byte_idx, buffer_size, &bytes_written, &ov) ==
           0 ||
       bytes_written != buffer_size) {
-    return LOG_STATUS(Status::IOError(std::string(
+    return LOG_STATUS(Status_IOError(std::string(
         "Cannot write to file; File writing error: " + get_last_error_msg())));
   }
   return Status::Ok();
 }
 
-std::string Win::uri_from_path(const std::string& path) {
-  if (path.length() == 0) {
-    return "";
-  }
-
-  unsigned long uri_length = INTERNET_MAX_URL_LENGTH;
-  char uri[INTERNET_MAX_URL_LENGTH];
-  std::string str_uri;
-  if (UrlCreateFromPath(path.c_str(), uri, &uri_length, 0) != S_OK) {
-    LOG_STATUS(Status::IOError(
-        std::string("Failed to convert path '" + path + "' to URI.")));
-  }
-  str_uri = uri;
-  return str_uri;
-}
-
-std::string Win::path_from_uri(const std::string& uri) {
-  if (uri.length() == 0) {
-    return "";
-  }
-
-  std::string uri_with_scheme =
-      (utils::parse::starts_with(uri, "file://") ||
-       // also accept 'file:/x...'
-       (utils::parse::starts_with(uri, "file:/") && uri.substr(6, 1) != "/")) ?
-          uri
-          // else treat as file: item on 'localhost' (empty host name)
-          :
-          "file:///" + uri;
-
-  unsigned long path_length = MAX_PATH;
-  char path[MAX_PATH];
-  std::string str_path;
-  if (PathCreateFromUrl(uri_with_scheme.c_str(), path, &path_length, 0) !=
-      S_OK) {
-    LOG_STATUS(Status::IOError(std::string(
-        "Failed to convert URI '" + uri_with_scheme + "' to path.")));
-  }
-  str_path = path;
-  return str_path;
-}
-
-bool Win::is_win_path(const std::string& path) {
-  if (path.empty()) {
-    // Special case to match the behavior of posix_filesystem.
-    return true;
-  } else if (PathIsURL(path.c_str())) {
-    return false;
-  } else {
-    bool definitely_windows = PathIsUNC(path.c_str()) ||
-                              PathGetDriveNumber(path.c_str()) != -1 ||
-                              path.find('\\') != std::string::npos;
-    if (definitely_windows) {
-      return true;
-    } else {
-      // Bare relative path e.g. "filename.txt"
-      return path.find('/') == std::string::npos;
-    }
-  }
-}
-
 }  // namespace sm
 }  // namespace tiledb
 
