@@ -122,6 +122,15 @@ int cell_validity_compression_level = -1;
 /** Special name reserved for the coordinates attribute. */
 const std::string coords = "__coords";
 
+/** Special name reserved for the timestamp attribute. */
+const std::string timestamps = "__timestamps";
+
+/** The size of a timestamp cell. */
+const uint64_t timestamp_size = sizeof(uint64_t);
+
+/** The type of a variable offset cell. */
+extern const Datatype timestamp_type = Datatype::UINT64;
+
 /** The default compressor for the coordinates. */
 Compressor coords_compression = Compressor::ZSTD;
 
@@ -149,6 +158,9 @@ const char empty_char = std::numeric_limits<char>::min();
 /** The special value for an empty blob. */
 constexpr std::byte empty_blob{0};
 
+/** The special value for an empty bool. */
+const uint8_t empty_bool = 0;
+
 /** The special value for an empty int8. */
 const int8_t empty_int8 = std::numeric_limits<int8_t>::min();
 
@@ -382,6 +394,9 @@ const std::string char_str = "CHAR";
 /** The string representation for type blob. */
 const std::string blob_str = "BLOB";
 
+/** The string representation for type bool. */
+const std::string bool_str = "BOOL";
+
 /** The string representation for type int8. */
 const std::string int8_str = "INT8";
 
@@ -555,11 +570,14 @@ const int32_t library_version[3] = {
     TILEDB_VERSION_MAJOR, TILEDB_VERSION_MINOR, TILEDB_VERSION_PATCH};
 
 /** The TileDB serialization format version number. */
-const uint32_t format_version = 13;
+const uint32_t format_version = 14;
 
 /** The lowest version supported for back compat writes. */
 const uint32_t back_compat_writes_min_format_version = 7;
 
+/** The lowest version supported for consolidation with timestamps. */
+const uint32_t consolidation_with_timestamps_min_version = 14;
+
 /** The maximum size of a tile chunk (unit of compression) in bytes. */
 const uint64_t max_tile_chunk_size = 64 * 1024;
 
@@ -640,6 +658,8 @@ const void* fill_value(Datatype type) {
   switch (type) {
     case Datatype::BLOB:
       return &constants::empty_blob;
+    case Datatype::BOOL:
+      return &constants::empty_bool;
     case Datatype::INT8:
       return &constants::empty_int8;
     case Datatype::UINT8:
