@@ -138,7 +138,7 @@ class Query {
    */
   Query(
       StorageManager* storage_manager,
-      Array* array,
+      shared_ptr<Array> array,
       URI fragment_uri = URI(""));
 
   /** Destructor. */
@@ -340,6 +340,11 @@ class Query {
   Status get_written_fragment_timestamp_range(
       uint32_t idx, uint64_t* t1, uint64_t* t2) const;
 
+  /** Returns the array's smart pointer. */
+  inline shared_ptr<Array> array_shared() {
+    return array_shared_;
+  }
+
   /** Returns the array. */
   const Array* array() const;
 
@@ -556,10 +561,15 @@ class Query {
   void clear_strategy();
 
   /**
-   * Disables checking the global order. Applicable only to writes.
-   * This option will supercede the config.
+   * Disables checking the global order and coordinate duplicates. Applicable
+   * only to writes. This option will supercede the config.
    */
-  Status disable_check_global_order();
+  Status disable_checks_consolidation();
+
+  /**
+   * Enables consolidation with timestamps.
+   */
+  Status set_consolidation_with_timestamps();
 
   /**
    * Sets the config for the Query
@@ -889,7 +899,12 @@ class Query {
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
-  /** The array the query is associated with. */
+  /** A smart pointer to the array the query is associated with.
+   * Ensures that the Array object exists as long as the Query object exists. */
+  shared_ptr<Array> array_shared_;
+
+  /** The array the query is associated with.
+   * Cached copy of array_shared_.get(). */
   Array* array_;
 
   /** The array schema. */
@@ -976,9 +991,14 @@ class Query {
 
   /**
    * If `true`, it will not check if the written coordinates are
-   * in the global order. This supercedes the config.
+   * in the global order or have duplicates. This supercedes the config.
    */
-  bool disable_check_global_order_;
+  bool disable_checks_consolidation_;
+
+  /**
+   * If `true`, it will enable consolidation with timestamps on the reader.
+   */
+  bool consolidation_with_timestamps_;
 
   /** The name of the new fragment to be created for writes. */
   URI fragment_uri_;
