@@ -165,9 +165,24 @@ class ResultTile {
   /**
    * Returns true if the coordinates at position `pos_a` and `pos_b` are
    * the same.
+   *
+   * This checks for two cells in different tiles.
    */
   bool same_coords(const ResultTile& rt, uint64_t pos_a, uint64_t pos_b) const;
 
+  /**
+   * Returns true if the coordinates at position `pos_a` and `pos_b` are
+   * the same.
+   *
+   * This checks for two cells in the same tile.
+   */
+  bool same_coords(uint64_t pos_a, uint64_t pos_b) const;
+
+  /**
+   * Returns the timestamp of the cell at position `pos`.
+   */
+  uint64_t timestamp(uint64_t pos);
+
   /** Returns the fragment id that this result tile belongs to. */
   unsigned frag_idx() const;
 
@@ -256,6 +271,29 @@ class ResultTile {
       const uint64_t min_cell,
       const uint64_t max_cell);
 
+  /**
+   * Applicable only to sparse arrays.
+   *
+   * Computes a result count for the input string dimension for the coordinates
+   * that fall in the input ranges and multiply with the previous count. The
+   * caller has already determined that the input start-end range needs to be
+   * processed fully.
+   *
+   * When called over multiple ranges, this follows the formula:
+   * total_count = d1_count * d2_count ... dN_count.
+   */
+  template <class BitmapType>
+  static void compute_results_count_sparse_string_range(
+      const std::vector<std::pair<std::string_view, std::string_view>>
+          cached_ranges,
+      const char* buff_str,
+      const uint64_t* buff_off,
+      const uint64_t cell_num,
+      const uint64_t buff_str_size,
+      const uint64_t start,
+      const uint64_t end,
+      std::vector<BitmapType>& result_count);
+
   /**
    * Applicable only to sparse arrays.
    *
@@ -349,6 +387,9 @@ class ResultTile {
   /** Attribute names to tiles based on attribute ordering from array schema. */
   std::vector<std::pair<std::string, optional<TileTuple>>> attr_tiles_;
 
+  /** The timestamp attribute tile. */
+  optional<TileTuple> timestamps_tile_;
+
   /** The zipped coordinates tile. */
   TileTuple coords_tile_;
 
@@ -630,6 +671,26 @@ class GlobalOrderResultTile : public ResultTileWithBitmap<uint8_t> {
     hilbert_values_[i] = v;
   }
 
+  /** Return first cell index in bitmap. */
+  uint64_t first_cell_in_bitmap() {
+    uint64_t ret = 0;
+    while (!bitmap_[ret]) {
+      ret++;
+    }
+
+    return ret;
+  }
+
+  /** Return first cell index in bitmap. */
+  uint64_t last_cell_in_bitmap() {
+    uint64_t ret = bitmap_.size() - 1;
+    while (!bitmap_[ret]) {
+      ret--;
+    }
+
+    return ret;
+  }
+
  private:
   /* ********************************* */
   /*        PRIVATE ATTRIBUTES         */
